@@ -589,7 +589,7 @@ void lk_karmaLan_load_database(int rio)
     char *settings;
 
     lk_karmaLan_get_device_settings(rio);
-    if(lk_properties_cache_obsolete(NULL)){ /* DB was changed */
+    if(lk_properties_cache_obsolete()){ /* DB was changed */
         lk_karmaLan_get_all_file_details(rio, &settings);
         lk_properties_import(settings);
         free(settings);
