@@ -9,7 +9,10 @@
  * the GNU GPL, version 2 or later.
  */
 
+#ifndef __APPLE__
 #define _XOPEN_SOURCE 500        /* Needed by pread/pwrite */
+#endif
+
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -18,12 +21,17 @@
 #include <string.h>
 #include <strings.h>
 #include <sys/stat.h>
-#include <sys/statfs.h>
-#include <sys/vfs.h>            /* or <sys/statfs.h> */
 #include <sys/types.h>
 #include <unistd.h>
 #include <utime.h>
 
+#ifdef __APPLE__
+#include <sys/mount.h>
+#else
+#include <sys/statfs.h>
+#include <sys/vfs.h>            /* or <sys/statfs.h> */
+#endif
+
 #include "lkarma.h"
 #include "karma.h"
 #include "karmaUsb.h"
@@ -74,9 +82,6 @@ int lk_karmaUsb_connect(char *path)
     strcpy(usbMountPoint, path);
     if(path[len - 1] != '/')
         strcat(usbMountPoint, "/");
-    ret = lk_is_karma_mountpoint(usbMountPoint);
-    if (ret)
-        return ret;
 
     lk_karmaUsb_get_device_settings(0);
 
@@ -125,7 +130,7 @@ int lk_karmaUsb_get_storage_details(int rio, uint32_t storage_id,
     aux = statfs(usbMountPoint, &buf);
     /*perror("+++statfs"); */
     if(aux == -1) {
-        lk_errors_set(E_FAILEDREQ);
+        lk_errors_set(E_NOMOUNT);
         return -1;
     }
 
@@ -220,7 +225,7 @@ static int read8(FILE * fp, uint32_t * dest)
 static
 int is_deleted(karma_db_t * db, uint32_t index)
 {
-    if(!db->dmap_size || (index >> 3) > db->dmap_size)
+    if((index >> 3) >= db->dmap_size)
         return 0;
     return db->dmap[index >> 3] & (1 << (index % 8));
 }
@@ -445,9 +450,14 @@ static int32_t read_properties(char **properties)
     strncpy(path1, usbMountPoint, 1023 - strlen("/fids0/_00000/100"));
     strcat(path1, "/fids0/");
     dp0 = opendir(path1);
-    if(dp0 == NULL) {
-        lk_errors_set(E_NODIR);
-        return -1;
+    if(!dp0) {
+        mk_path(path1);
+        dp0 = opendir(path1);
+    
+        if (!dp0) {
+            lk_errors_set(E_NODIR);
+            return -1;
+        }
     }
     while((de0 = readdir(dp0)) != NULL) {
         if(strncmp(de0->d_name, "_0", 2) != 0)
@@ -741,8 +751,6 @@ static int32_t read_properties_smalldb(char **properties)
     }
     *ptr = '\0';
     *properties = tmp;
-    db->dmap_size = 0;
-    db->num_playlists = 0;
 
   out:
     ret = 0;
@@ -755,6 +763,8 @@ static int32_t read_properties_smalldb(char **properties)
         free(prop_strs);
     if (db)
     {
+        db->dmap_size = 0;
+        db->num_playlists = 0;
         if (db->dmap)
             free(db->dmap);
         if (db->playlists)
@@ -820,21 +830,22 @@ int32_t lk_karmaUsb_write_file_chunk(int rio, uint64_t offset, uint64_t size,
     char *path = NULL;
 
     path = lk_karmaUsb_fidToPath(rio, file_id);
+
 /*
     if(offset==0)
         fprintf(stderr, "--> %s %x %d\n", path, file_id, file_id);
 */
     if((fd = open(path, O_WRONLY | O_CREAT, PERMS)) == EEXIST)
         fd = open(path, O_WRONLY | O_TRUNC, PERMS);
-    else if(fd == -1 && errno == ENOENT) {
-        res = strlen(path);
-        path[res - 4] = '\0';
-        ret = mkdir(path, 0755);
-        path[res - 4] = '/';
-        if(!ret)
-            if((fd = open(path, O_WRONLY | O_CREAT, PERMS)) == EEXIST)
-                fd = open(path, O_WRONLY | O_TRUNC, PERMS);
+
+    /* make sure the directory is there and try again */
+    if (fd == -1) {
+        mk_path(path);
+
+        if((fd = open(path, O_WRONLY | O_CREAT, PERMS)) == EEXIST)
+            fd = open(path, O_WRONLY | O_TRUNC, PERMS);
     }
+    
     free(path);
 /*  fprintf(stderr, "---> %s\n", strerror(errno)); */
     if(fd == -1) {
@@ -966,10 +977,8 @@ void lk_karmaUsb_load_database_smalldb(int rio)
     aux = read_properties_smalldb(&properties);
     if(properties)
         count = lk_properties_import(properties);
-    else {
-        lk_errors_set(E_FAILEDREQ);
-        return;
-    }
+    /* else just assume a new blank DB */
+    
     lk_fdb_load(1);
 /*  fprintf(stderr, "smalldb: %d Files\n", count); */
     if(properties)
