@@ -18,6 +18,7 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <limits.h>
+#include <ctype.h>
 
 #include <taglib/tag_c.h>
 
@@ -216,11 +217,12 @@ static char *lk_rio_create_temp_file(int rio, uint32_t fid)
     if (ret != 0 || got != blk)
         goto err;
 
-    len = strlen(karma_tmpdir) + strlen(simple_itoa(fid)) + 8;
-    filename = malloc(len);
-    snprintf(filename, len, "%s/lkarma%d", karma_tmpdir, fid);
+    len = strlen(karma_tmpdir) + strlen("/lkarmaXXXXXX");
+    filename = malloc(len+1);
+    strcpy(filename, karma_tmpdir);
+    strcat(filename, "/lkarmaXXXXXX");
 
-    fd = open(filename, O_WRONLY|O_CREAT|O_EXCL, 0600);
+    fd = mkstemp(filename); /* filename X's are replaced with a unique string */
     if (fd == -1)
         goto err;
 
@@ -361,14 +363,16 @@ static uint32_t lk_rio_do_write(int fdb, int rio, const char * filename)
             free(ids);
             lk_errors_set(E_DUPE);
             lk_properties_del_property(fid);
-	    return -1;
+            close(fd);
+            return -1;
 	}
     }
 
-    while((got=read(fd, &tmp, siz)) > 0) {  /* Now writting the data */
+    while((got=read(fd, &tmp, siz)) > 0) {  /* Now writing the data */
         if(lk_karma_write_file_chunk(rio, offs, got, fid, 0, tmp)!=0) {
 	    lk_errors_set(E_WRCHUNK);
             lk_properties_del_property(fid);
+            close(fd);
             return -1;
         }
         offs+=got;
@@ -543,6 +547,15 @@ static void add_tag(HASH *p, char *key, char *value)
     /* printf("Key = %s ; Value = %s\n", key, value); */
 }
 
+char *strrtrim (char *s) {
+    size_t n;
+    if(!s) return NULL;
+    n = strlen(s) - 1;
+    while (n && (isspace(s[n])))
+        n--;
+    s[n+1] = '\0';
+    return s;
+}
 /*** You can use this with all tag-formats, which taglib supports.
   */
 static int set_tag_props(HASH *p, const char *filename, int type)
@@ -582,6 +595,10 @@ static int set_tag_props(HASH *p, const char *filename, int type)
     title  = taglib_tag_title (tl_tag);
     source = taglib_tag_album (tl_tag);
     artist = taglib_tag_artist(tl_tag);
+    title  = strrtrim(title);
+    source = strrtrim(source);
+    artist = strrtrim(artist);
+/*  fprintf(stderr, "set_tag_props -%s-%s-%s-\n", artist, source, title); */
 
     if (write_dupes == 0) {
         ids = lk_properties_andOrSearch(EXACT|ORS, NULL, "title", title);
