@@ -48,7 +48,7 @@ char *expand(char *str);
 void downloadFids(uint32_t *vp, int doDownload, char *unkName,
                   char* useTuneProps, char* useTaxiProps, char* usePlstProps,
                   char* edPathStr1, char* edPathStr2, char* edPathStr3,
-                  int squeezeRep);
+                  int squeezeRep, int expandPlaylists, char *playListName);
 
 
 void usage(int complete)
@@ -57,8 +57,8 @@ void usage(int complete)
                         "[-s searchString ]\n"
            "             [-u props] [-x props] [-l props] "
                         "[-c charSet] [-e set1 set2 set3]\n"
-           "             [-D dummyName] [-q] [-d] [-r] [-R] [-b] [-w] [-f]\n" 
-           "             [-F] [-H] [-U fdb.gz] [files_to_upload ...]\n"
+           "             [-D dummyName] [-U fdb.gz] [-b] [-d] [-f] [-F]\n"
+           "             [-H] [-L] [-q] [-r] [-R] [-w] [files_to_upload ...]\n"
            "\n");
     if(!complete) return;
     printf("  \"-s\" AND/OR-search karma DB according to a \"searchString\";\n"
@@ -76,11 +76,12 @@ void usage(int complete)
            " Default=\'%%f.%%c\'\n"
            "  \"-x\" like -u for <type>=\'taxi\',.    Default=\"-u props\"\n"
            "  \"-l\" like -u for <type>=\'playlist\'. Default=\"-x props\"\n");
-    printf("       \"props\" are formating strings (as in \"printf\"), where\n"
+    printf("       \"props\" are formatting strings (as in \"printf\"), where\n"
            "       \'%%T\', \'%%g\', \'%%a\', \'%%s\', \'%%f\', \'%%t\', "
            "\'%%n\', \'%%c\' are, resp., values of\n"
            "       <type>, <genre>, <artist>, <source>, <fid>, <title>, "
            "<tracknr>, <codec>\n"
+           "       If -L enabled, \'%%l\' is the <playListName> in -l props\n"
            "\n");
     printf("  \"-e\" edit the download pathname using \"set1\" and "
            "\"set2\" as in unix \'tr\'\n"
@@ -101,6 +102,8 @@ void usage(int complete)
            " for download\n");
     printf("  \"-H\" Download files listed in ~/.openrio/.../" FDB_FILENAME
            "\n       that match the current hostname\n");
+    printf("  \"-L\" For playlists, print/download the listed tunes, "
+           "rather than the lists\n");
     printf("  \"-U\" Upload files listed in the given fdb.gz file\n");
 }
 
@@ -247,17 +250,42 @@ void deleteFids(uint32_t *vp) {
 /* ------------------ Pathname editing: now in pathedit.c ------------------- */
 /* -------------------------------------------------------------------------- */
 
+static uint32_t *addPlayListContents( uint32_t PLfid, uint32_t *previousFids) {
+    int i, j, count;
+    uint32_t fid, fidGeneration, *vp = previousFids;
+    playlist *pl;
+    
+    pl = lk_playlist_fid_read(PLfid);  
+    count = lk_playlist_count(pl);
+    printf("#---- Contents of playlist \"%s\" (fid=%d, %d entries)\n", 
+                                               pl->name, pl->fid, count);
+    j=0;
+    if(!vp) {
+        vp = malloc(count*sizeof(uint32_t)+1);  
+    } else {
+        while(vp[j]!=0)
+	    j++;
+        vp = realloc(vp, (j+count)*sizeof(uint32_t)+1);
+    }
+    for (i=0; i<count; i++, j++) {
+        lk_playlist_get_item(pl, i, &fid, &fidGeneration);
+        vp[j] = fid;
+    }
+    vp[j] = 0;
+    return vp;
+}
+
 void downloadFids(uint32_t *vp, int doDownload, char *unkName,
                   char* useTuneProps, char* useTaxiProps, char* usePlstProps, 
                   char* edPathStr1, char* edPathStr2, char* edPathStr3, 
-                  int squeezeRep)
+                  int squeezeRep, int expandPlaylists, char *playListName)
 {
 #define FLEN 512
     int karma, i, j, len, pos;
     char c, cc, *useProps=NULL, filename[FLEN];
     char *type, *genre, *artist,*source, *title, *tracknr, *codec;
     char *s;
-    uint32_t fid;
+    uint32_t fid, *vl;
 
 # define STRCAT(a,b) do { \
         s = ((b&&strlen(b))?b:unkName); \
@@ -278,8 +306,20 @@ void downloadFids(uint32_t *vp, int doDownload, char *unkName,
             fprintf(stderr, "**Could not get type of fid=%d\n", fid);
             return;
         }
-        if(strcmp((char *)type, "playlist") == 0)
-            useProps=strdup(usePlstProps);
+        if(strcmp((char *)type, "playlist") == 0) {
+            if(expandPlaylists) {
+		vl = addPlayListContents(fid, NULL);
+ 	        title = lk_properties_get_property(fid, "title");
+		downloadFids(vl,           doDownload,   unkName,
+                             usePlstProps, usePlstProps, usePlstProps, 
+                             edPathStr1,   edPathStr2,   edPathStr3, 
+                             squeezeRep,   expandPlaylists, title);
+		free(vl);
+                continue;
+	    }
+	    else
+	        useProps=strdup(usePlstProps);
+	}
         if(strcmp((char *)type, "taxi") == 0)
             useProps=strdup(useTaxiProps);
         if(strcmp((char *)type, "tune") == 0)
@@ -359,6 +399,10 @@ void downloadFids(uint32_t *vp, int doDownload, char *unkName,
                         }
                     }
                     break;
+                case 'l':                           /* playlist name */
+                    if(expandPlaylists && playListName)
+		         STRCAT(filename, playListName);
+                    break;
                 default:
                     fprintf(stderr, "* Unknown property \'%c\' - ignored\n", c);
                     break;
@@ -408,6 +452,7 @@ double getUsedSize(int karma)
     return(size - freeSpace);
 }
 
+/* ************************************************************************** */
 int main(int argc, char * argv[])
 {
     int i, c, r;
@@ -423,6 +468,7 @@ int main(int argc, char * argv[])
     char *editPathStr2=NULL;
     char *editPathStr3=NULL;
     char *unkName=UNK_NAME;          /* Dummy name for missing properties */
+    int  expandPlaylists=0;
     int  squeezeRepeats=0;
     int  download=0;
     int  fdbDownload=0;
@@ -441,7 +487,8 @@ int main(int argc, char * argv[])
     char *device=NULL;
     char *fdb=NULL;
     
-    while ((c = getopt(argc, argv, "dhqrRbwfFHa:c:e:p:s:S:u:x:l:D:U:")) != -1) {
+    while ((c = getopt(argc, argv, "dhqrRbwfFHLa:c:e:p:s:S:u:x:l:D:U:")) 
+                                                                   != -1) {
         switch(c) {
             case 'h': case '?':
                 usage(1);
@@ -510,6 +557,9 @@ int main(int argc, char * argv[])
             case 'D':
                 unkName=strdup(optarg);
                 break;
+            case 'L':
+		expandPlaylists = 1;
+                break;
             case 'U':
                 fdb=strdup(optarg);
                 break;
@@ -614,7 +664,7 @@ int main(int argc, char * argv[])
     }
 
     lk_karma_get_device_settings(karma, &settings);
-    dbChanged = lk_properties_cache_obsolete(settings);
+    dbChanged = lk_properties_cache_obsolete();
     if(dbChanged) {
         fprintf(stderr, "Downloading Rio database... ");
         fflush(stdout);
@@ -634,13 +684,13 @@ int main(int argc, char * argv[])
         vp=searchStr(exact, utf, unkName);
         free(utf);
     }
+
     if(vp)
         downloadFids(vp, download, unkName, tunePropsToUse, taxiPropsToUse,
                      plstPropsToUse, editPathStr1, editPathStr2, editPathStr3,
-                     squeezeRepeats);
-    
-    if(taxiPropsToUse) free(taxiPropsToUse); /*Enrique?*/
-    if(plstPropsToUse) free(plstPropsToUse); /*Enrique?*/
+                     squeezeRepeats, expandPlaylists, NULL);
+    if(taxiPropsToUse) free(taxiPropsToUse);
+    if(plstPropsToUse) free(plstPropsToUse);
 
     if(removeFiles>0 && vp) {
         yesNot='Y';
